function [std_pi,std_y,std_g,std_piW,std_w,std_i,std_phiw,mean_pi,mean_y,mean_g,mean_piW,mean_w,mean_i,mean_phiw,lbb,simloss,maxima,minima] = NKMsimul(cp,cy,cg,cpW,cw,ci,cphiw,fspace,T,buin,nobs,params,cufo)

% Function file to simulate New Keynesian model with ZLB

% Subroutines called:
% funeval (from CompEcon)

cd ..
load simul_d_cp.mat;
cd(cufo);
deps = d_inno'; clear d_inno;
ueps = u_inno'; clear u_inno;

beta=params.beta;
rho = params.rho;
lambda = params.lambda;
lambdaGtrue = params.lambdaGtrue;
lambdaW = params.lambdaW;
Gamma = params.Gamma;
rhou = params.rhou;

pi = zeros(nobs,T+buin);
y = zeros(nobs,T+buin);
g = zeros(nobs,T+buin);
piW = zeros(nobs,T+buin);
w = zeros(nobs,T+buin);
i = zeros(nobs,T+buin);
phiw = zeros(nobs,T+buin);
w0 = zeros(nobs,1);

d = zeros(nobs,T+buin);
u = zeros(nobs,T+buin);

simloss = zeros(nobs,T);
lbindex = zeros(nobs,T);

betaadd = 1;

d(:,1) = deps(:,1);
u(:,1) = ueps(:,1);
w(:,1) = funeval(cw,fspace,[d(:,1) w0 u(:,1)]);

for j = 2:1:T+buin;
    d(:,j)=rho*d(:,j-1)+deps(:,j);
    u(:,j)=rhou*u(:,j-1)+ueps(:,j);
    pi(:,j) = funeval(cp,fspace,[d(:,j) w(:,j-1) u(:,j)]);
    y(:,j) = funeval(cy,fspace,[d(:,j) w(:,j-1) u(:,j)]);
    g(:,j) = funeval(cg,fspace,[d(:,j) w(:,j-1) u(:,j)]);
    piW(:,j) = funeval(cpW,fspace,[d(:,j) w(:,j-1) u(:,j)]);
    w(:,j) = funeval(cw,fspace,[d(:,j) w(:,j-1) u(:,j)]);
    i(:,j) = funeval(ci,fspace,[d(:,j) w(:,j-1) u(:,j)]);
    phiw(:,j) = funeval(cphiw,fspace,[d(:,j) w(:,j-1) u(:,j)]);

    
    if j > buin;   
        simloss(:,j-buin) = betaadd*(pi(:,j).^(2)+lambda*(y(:,j)-Gamma*g(:,j)).^(2)+lambdaGtrue*g(:,j).^(2)+lambdaW*piW(:,j).^(2));
        betaadd = betaadd*beta;
        
        [row,col] = find(i(:,j)<=0.0000000001);
        lbindex(row,j) = 1;
    end;
end;

lbb = sum(lbindex,2)*1/T;

varpi = var(pi(:,end-T+1:end),0,2);
vary = var(y(:,end-T+1:end),0,2);
varg = var(g(:,end-T+1:end),0,2);
varpiW = var(piW(:,end-T+1:end),0,2);
varw = var(w(:,end-T+1:end),0,2);
vari = var(i(:,end-T+1:end),0,2);
varphiw = var(phiw(:,end-T+1:end),0,2);


std_pi = sqrt(varpi);
std_y = sqrt(vary);
std_g = sqrt(varg);
std_piW = sqrt(varpiW);
std_w = sqrt(varw);
std_i = sqrt(vari);
std_phiw = sqrt(varphiw);

mean_pi = mean(pi(:,end-T+1:end),2);
mean_y = mean(y(:,end-T+1:end),2);
mean_g = mean(g(:,end-T+1:end),2);
mean_piW = mean(piW(:,end-T+1:end),2);
mean_w = mean(w(:,end-T+1:end),2);
mean_i = mean(i(:,end-T+1:end),2);
mean_phiw = mean(phiw(:,end-T+1:end),2);


maxima = max(w(:,end-T+1:end));
minima = min(w(:,end-T+1:end));

simloss = sum(simloss,2);