% 'Fiscal Activism and the Zero Nominal Interest Rate Bound'
% Sebastian Schmidt, sebastian.schmidt@ecb.int

% Optimal delegation case

% CompEcon toolbox: fundefn, funnode, qnwnorm, funeval

clear all;
clc;
close all;
format long;

% addpath('D:\sebastian\COMPeconToolbox\CEtools');
addpath('\\GIMECB01\HOMEDIR-QU$\schmids\Research Sebastian\COMPeconToolbox\CEtools');

cufo = pwd;

maxit = 50000;

% Parameters
beta = 0.9938;  
rstar = 100*(1/beta-1);
GY = 0.2;
alpha = 0.72; 
theta = 9; 
sigma = 1.22;
omega = sigma*(1-GY)/GY;  
eta = 1.69;
gamma = 0.3; 
Gamma = sigma/(sigma+eta/(1-gamma));
kappap = (1-alpha)*(1-alpha*beta)/alpha*(1-gamma)/(1-gamma+gamma*theta);
kappa = kappap*(sigma+eta);
alphaw = 0.72; 
thetaw = 9; 
kappaw = (1-beta*alphaw)*(1-alphaw)/(alphaw*(1+eta*thetaw));
lambda = kappap*(sigma+(eta+gamma)/(1-gamma))*1/theta;
lambdaG = 0.004;
lambdaGtrue = lambda*Gamma*(1-Gamma+omega/sigma);
lambdaW = lambda*(1-gamma)*thetaw/(kappaw*(sigma+(eta+gamma)/(1-gamma)));

rho = 0.8; 
stdevd = 0.2975; 
uncovad = stdevd^2/(1-rho^2);
uncostdevd = sqrt(uncovad);
rhou = 0; 
stdevu = 0.17; 
uncovau = stdevu^2/(1-rhou^2);
uncostdevu = sqrt(uncovau);


params.rstar = rstar;
params.beta = beta;
params.sigma = sigma;
params.eta = eta;
params.gamma = gamma;
params.Gamma = Gamma;
params.kappap = kappap;
params.kappaw = kappaw;
params.lambda = lambda;
params.lambdaG = lambdaG;
params.lambdaGtrue = lambdaGtrue;
params.lambdaW = lambdaW;
params.rho = rho;
params.uncostdevd = uncostdevd;
params.stdevd = stdevd;
params.rhou = rhou;
params.uncostdevu = uncostdevu;
params.stdevu = stdevu;


% Collocation
% State variables: natural real rate shock, real wage rate, price mark-up shock
 
smin   = [-uncostdevd*4 -1.78 -uncostdevu*4];     
smax   = [uncostdevd*4 1.38 uncostdevu*4];
dvec = [linspace(-uncostdevd*4,0,17)'; linspace(0.1,uncostdevd*4,8)'];
wvec = linspace(-1.78,1.38,19)';
uvec = linspace(-uncostdevu*4,uncostdevu*4,9)';
fspace = fundefn('spli',[],[],[],3,dvec,wvec,uvec);

tol = 1.49*10^(-8); % tolerance crition for iteration

scoord = funnode(fspace); % collocation nodes
snodes = gridmake(scoord);
Phi = funbas(fspace,snodes);

upd = 0.5; % Updating parameter to avoid overshooting
up = 0.2; 

% Gaussian Quadrature 
[epsilons,ws] = qnwnorm([7 3],[0 0], [stdevd^2 0; 0 stdevu^2]); 

% Initial guess
pi = 0.119773*snodes(:,2) +0.868425*snodes(:,3);
y = -0.003282*snodes(:,2) -1.035766*snodes(:,3);
g = -0.012227*snodes(:,2) -0.164531*snodes(:,3);
piW = -0.035821*snodes(:,2) +0.012102*snodes(:,3);
w = 0.844406*snodes(:,2) -0.856323*snodes(:,3);
i = rstar*ones(length(snodes),1) +1.22*snodes(:,1) +0.099439*snodes(:,2) +0.950997*snodes(:,3);
phiw = 1/(kappaw*(sigma+eta/(1-gamma))-kappap*gamma/(1-gamma))*(lambda*(y-Gamma*g)+kappap*gamma/(1-gamma)*pi+kappaw*(sigma+eta/(1-gamma))*lambdaW*piW);

cp = Phi\pi;
cy = Phi\y;
cg = Phi\g;
cpW = Phi\piW;
cw = Phi\w;
ci = Phi\i;
cphiw = Phi\phiw;

Ep_w = 0.119773*ones(length(snodes),1); 
Ey_w = -0.003282*ones(length(snodes),1); 
Eg_w = -0.012227*ones(length(snodes),1);
EpW_w = -0.035821*ones(length(snodes),1); 

      
Ep_wnew = zeros(length(snodes),1);
Ey_wnew = zeros(length(snodes),1);
Eg_wnew = zeros(length(snodes),1);
EpW_wnew = zeros(length(snodes),1);
Ep_wold = 1000*ones(length(snodes),1);
Ey_wold = 1000*ones(length(snodes),1);
Eg_wold = 1000*ones(length(snodes),1);
EpW_wold = 1000*ones(length(snodes),1);

load polfuncoef_zlb.mat;

while norm([Ep_wnew-Ep_wold;Ey_wnew-Ey_wold;Eg_wnew-Eg_wold;EpW_wnew-EpW_wold],inf) > 1.49*10^(-8);

changepartial = (norm([Ep_wnew-Ep_wold;Ey_wnew-Ey_wold;Eg_wnew-Eg_wold;EpW_wnew-EpW_wold],inf));
   
Ep_wold = Ep_w;
Ey_wold = Ey_w;
Eg_wold = Eg_w;
EpW_wold = EpW_w;

    
stopall = 0;
for it = 1:maxit;
    cpold = cp;
    cyold = cy;
    cgold = cg;
    cpWold = cpW;
    cwold = cw;
    ciold = ci;
    cphiwold = cphiw;
    
    [pi,y,g,piW,w,i,phiw] = NKMsolve(snodes,cp,cy,cg,cpW,cw,ci,cphiw,epsilons,ws,fspace,params,Ep_w,Ey_w,Eg_w,EpW_w);
                                    

    % Update policy functions (Note, that I use linear splines)
    cpnew = Phi\pi;
    cynew = Phi\y;
    cgnew = Phi\g;
    cpWnew = Phi\piW;
    cwnew = Phi\w;
    cinew = Phi\i;
    cphiwnew = Phi\phiw;
    
    cp = upd*cpnew+(1-upd)*cpold;
    cy = upd*cynew+(1-upd)*cyold;
    cg = upd*cgnew+(1-upd)*cgold;
    cpW = upd*cpWnew+(1-upd)*cpWold;
    cw = upd*cwnew+(1-upd)*cwold;
    ci = upd*cinew+(1-upd)*ciold;
    cphiw = upd*cphiwnew+(1-upd)*cphiwold;
    
    change = norm([cpnew-cpold;cynew-cyold;cgnew-cgold;cpWnew-cpWold;cwnew-cwold;cinew-ciold;cphiwnew-cphiwold],inf);
     disp([changepartial change]);

    if change > 50, disp('Algorithm does not converge.'), stopall = 1; break, end;
    if isnan(change) ==1, disp('Algorithm does not converge.'), stopall = 1; break, end;
    
    if change < tol, break, end;
    
end;

Ep_w = zeros(length(snodes),1);
Ey_w = zeros(length(snodes),1);
Eg_w = zeros(length(snodes),1);
EpW_w = zeros(length(snodes),1);

waux = funeval(cw,fspace,snodes);

for k=1:1:length(ws);
    df = rho*snodes(:,1)+ones(length(snodes),1)*epsilons(k,1);
    uf = rhou*snodes(:,3)+ones(length(snodes),1)*epsilons(k,2);
    Ep_w = Ep_w + ws(k)*funeval(cp,fspace,[df waux uf],[0 1 0]); % (cp[d(j),i(j+1)]-cp[d(j),i(j)])/h_i
    Ey_w = Ey_w + ws(k)*funeval(cy,fspace,[df waux uf],[0 1 0]);
    Eg_w = Eg_w + ws(k)*funeval(cg,fspace,[df waux uf],[0 1 0]);
    EpW_w = EpW_w + ws(k)*funeval(cpW,fspace,[df waux uf],[0 1 0]);
end;

Ep_wnew = Ep_w;
Ey_wnew = Ey_w;
Eg_wnew = Eg_w;
EpW_wnew = EpW_w;

Ep_w = up*Ep_wnew+(1-up)*Ep_wold;    
Ey_w = up*Ey_wnew+(1-up)*Ey_wold;
Eg_w = up*Eg_wnew+(1-up)*Eg_wold;
EpW_w = up*EpW_wnew+(1-up)*EpW_wold;
end;

save polfuncoef_zlb.mat cp cy cg cpW cw ci cphiw Ep_w Ey_w Eg_w EpW_w;

if stopall == 0;
    
% Impulse responses
NKMirf_d_forecast(cp,cy,cg,cpW,cw,ci,cphiw,fspace,39,params,ws,epsilons,cufo);

    
%% Simulation 
nobs = 2000;
T = 1000; 
buin = 50;

[std_pi,std_y,std_g,std_piW,std_w,std_i,std_phiw,...
    mean_pi,mean_y,mean_g,mean_piW,mean_w,mean_i,mean_phiw,...
    lobo,simloss,maxima,minima] = ...
NKMsimul(cp,cy,cg,cpW,cw,ci,cphiw,fspace,T,buin,nobs,params,cufo);


sd_pi = mean(std_pi);
sd_y = mean(std_y);
sd_g = mean(std_g);
sd_piW = mean(std_piW);
sd_w = mean(std_w);
sd_i = mean(std_i);
sd_phiw = mean(std_phiw);
zlbfreq = (mean(lobo))*100;
m_pi = mean(mean_pi);
m_y = mean(mean_y);
m_g = mean(mean_g);
m_piW = mean(mean_piW);
m_w = mean(mean_w);
m_i = mean(mean_i);
m_phiw = mean(mean_phiw);
miniw = min(minima);
maxiw = max(maxima);


loss = mean(simloss);
% Loss in percent of steady state private consumption 
% Remark I: Devide LOSS by 100^(2) because inflation and output gap are
% expressed in percentage terms.
% Remark II: Multiply by 100 because we want to express the loss in
% percentage terms
Closs = (1/2)*(1-beta)*1/lambda*(sigma+(eta+gamma)/(1-gamma))*loss/(100^(2))*100; 

% Equilibrium responses
NKMeqresp_d(cp,cy,cg,cpW,cw,ci,cphiw,fspace,params,ws,epsilons,m_w);
NKMeqresp_u(cp,cy,cg,cpW,cw,ci,cphiw,fspace,params,ws,epsilons,m_w);

% Risky steady state
NKMrss(cp,cy,cg,cpW,cw,ci,cphiw,fspace,5000,params,ws,epsilons);

save Results_delegation_cp.mat Closs loss zlbfreq sd_pi sd_y sd_g sd_piW sd_w sd_i sd_phiw m_pi m_y m_g m_piW m_w m_i m_phiw maxiw miniw 

disp('END');
end;
