function [pi,y,g,piW,w,i,phiw] = NKMsolve(s,cp,cy,cg,cpW,cw,ci,cphiw,epsilons,ws,fspace,params,Ep_w,Ey_w,Eg_w,EpW_w)

% Function file to solve New Keynesian model with ZLB using Collocation
% method

% Subroutines called:
% funeval (from CompEcon)

rho = params.rho;
rstar = params.rstar;
beta=params.beta;
sigma = params.sigma;
eta = params.eta;
gamma = params.gamma;
Gamma = params.Gamma;
kappap = params.kappap;
kappaw = params.kappaw;
lambda = params.lambda;
lambdaG = params.lambdaG;
lambdaW = params.lambdaW;

% Use results from previous iteration to initialize real wage w
w = funeval(cw,fspace,s);
pi = zeros(length(s),1);
y = zeros(length(s),1);
g = zeros(length(s),1);
piW = zeros(length(s),1);
i = zeros(length(s),1);
phiw = zeros(length(s),1);
    
 % Initialixe epectations functions
Epi = zeros(length(s),1);
Ey = zeros(length(s),1);
Eg = zeros(length(s),1);
EpiW = zeros(length(s),1);
Ephiw = zeros(length(s),1);

for k=1:1:length(ws);
    df = rho*s(:,1)+ones(length(s),1)*epsilons(k,1);
    Epi = Epi + ws(k)*funeval(cp,fspace,[df w]);
    Ey = Ey + ws(k)*funeval(cy,fspace,[df w]);
    Eg = Eg + ws(k)*funeval(cg,fspace,[df w]);
    EpiW = EpiW + ws(k)*funeval(cpW,fspace,[df w]);
    Ephiw = Ephiw + ws(k)*funeval(cphiw,fspace,[df w]);
end;


% 7x1 (constants)
B = [0; 1/sigma*rstar; 0; 0; 0; 0; 0];

% 7x1 (predetermined endogenous variables)
C = [0; 0; 0; -1; 0; 0; 0];

% 7x5 (expectations)
D = [beta 0 0 0 0;
    1/sigma 1 -1 0 0;
    0 0 0 beta 0;
    0 0 0 0 0;
    0 0 0 0 0;
    0 0 0 0 beta;
    0 0 0 0 0];

% 7x1 (demand shock)
E = [0; 1; 0; 0; 0; 0; 0];

for j =1:1:length(s);
    OM1 = Ey_w(j)-Eg_w(j)+1/sigma*Ep_w(j);

% 7x7 (free endogenous variables: pi, y, g, piW, w, i, phiw)
    A = [1 -kappap*gamma/(1-gamma) 0 0 -kappap 0 0;
         0 1 -1 0 0 1/sigma 0;
         0 -kappaw*(sigma+eta/(1-gamma)) Gamma*kappaw*(sigma+eta/(1-gamma)) 1 kappaw 0 0;
         -1 0 0 1 -1 0 0;
         kappap*gamma/(1-gamma) (1-Gamma)*lambda -(1-Gamma)*lambda*Gamma+lambdaG (1-Gamma)*kappaw*(sigma+eta/(1-gamma))*lambdaW 0 0 kappap*gamma/(1-gamma)-(1-Gamma)*kappaw*(sigma+eta/(1-gamma));
         beta*Ep_w(j)+kappap+OM1*kappap*gamma/(1-gamma) OM1*lambda -OM1*lambda*Gamma beta*lambdaW*EpW_w(j)-kappaw*lambdaW+OM1*kappaw*(sigma+eta/(1-gamma))*lambdaW 0 0 beta*Ep_w(j)-beta*EpW_w(j)+kappap+kappaw+1+OM1*(kappap*gamma/(1-gamma)-kappaw*(sigma+eta/(1-gamma)));
         kappap*gamma/(1-gamma) lambda -lambda*Gamma kappaw*(sigma+eta/(1-gamma))*lambdaW 0 0 kappap*gamma/(1-gamma)-kappaw*(sigma+eta/(1-gamma))];
        
    
    z = A\B + A\C*s(j,2) + A\D*[Epi(j); Ey(j); Eg(j); EpiW(j); Ephiw(j)] + A\E*s(j,1);
    pi(j)=z(1);
    y(j)=z(2);
    g(j)=z(3);
    piW(j) = z(4);
    w(j) = z(5);
    i(j) = z(6);
    phiw(j) = z(7);
    
    if i(j)<0;     
    Azlb = [1 -kappap*gamma/(1-gamma) 0 0 -kappap 0 0;
            0 1 -1 0 0 1/sigma 0;
            0 -kappaw*(sigma+eta/(1-gamma)) Gamma*kappaw*(sigma+eta/(1-gamma)) 1 kappaw 0 0;
            -1 0 0 1 -1 0 0;
            kappap*gamma/(1-gamma) (1-Gamma)*lambda -(1-Gamma)*lambda*Gamma+lambdaG (1-Gamma)*kappaw*(sigma+eta/(1-gamma))*lambdaW 0 0 kappap*gamma/(1-gamma)-(1-Gamma)*kappaw*(sigma+eta/(1-gamma));
            beta*Ep_w(j)+kappap+OM1*kappap*gamma/(1-gamma) OM1*lambda -OM1*lambda*Gamma beta*lambdaW*EpW_w(j)-kappaw*lambdaW+OM1*kappaw*(sigma+eta/(1-gamma))*lambdaW 0 0 beta*Ep_w(j)-beta*EpW_w(j)+kappap+kappaw+1+OM1*(kappap*gamma/(1-gamma)-kappaw*(sigma+eta/(1-gamma)));
            0 0 0 0 0 1 0];
        
        z = Azlb\B + Azlb\C*s(j,2) + Azlb\D*[Epi(j); Ey(j); Eg(j); EpiW(j); Ephiw(j)] + Azlb\E*s(j,1);
        pi(j)=z(1);
        y(j)=z(2);
        g(j)=z(3);
        piW(j) = z(4);
        w(j) = z(5);
        i(j) = z(6);
        phiw(j) = z(7);
    end;
    
end;






    
    
    
    
    
    
    






