% 'Fiscal Activism and the Zero Nominal Interest Rate Bound'
% Sebastian Schmidt, sebastian.schmidt@ecb.int

% Grid search for optimal degree of fiscal activism
% Model with ZLB and natural rate shock, without price mark-up shock

% CompEcon toolbox: fundefn, funnode, qnwnorm, funeval

clear all;
clc;
close all;
format long;

% addpath('D:\sebastian\COMPeconToolbox\CEtools');
addpath('\\GIMECB01\HOMEDIR-QU$\schmids\Research Sebastian\COMPeconToolbox\CEtools');

cufo = pwd;

maxit = 50000;

% Parameters
beta = 0.9938; 
rstar = 100*(1/beta-1);
GY = 0.2;
alpha = 0.72; 
theta = 9; 
sigma = 1.22;
omega = sigma*(1-GY)/GY;  
eta = 1.69;
gamma = 0.3; 
Gamma = sigma/(sigma+eta/(1-gamma));
kappap = (1-alpha)*(1-alpha*beta)/alpha*(1-gamma)/(1-gamma+gamma*theta);
kappa = kappap*(sigma+eta);
alphaw = 0.72; 
thetaw = 9; 
kappaw = (1-beta*alphaw)*(1-alphaw)/(alphaw*(1+eta*thetaw));
lambda = kappap*(sigma+(eta+gamma)/(1-gamma))*1/theta;
lambdaGstar = lambda*Gamma*(1-Gamma+omega/sigma); % society's weight
lambdaW = lambda*(1-gamma)*thetaw/(kappaw*(sigma+(eta+gamma)/(1-gamma)));

rho = 0.8; 
stdevd = 0.2975; 
uncovad = stdevd^2/(1-rho^2);
uncostdevd = sqrt(uncovad);


params.rstar = rstar;
params.beta = beta;
params.sigma = sigma;
params.eta = eta;
params.gamma = gamma;
params.Gamma = Gamma;
params.kappap = kappap;
params.kappaw = kappaw;
params.lambda = lambda;
params.lambdaGstar = lambdaGstar;
params.lambdaW = lambdaW;
params.rho = rho;
params.uncostdevd = uncostdevd;
params.stdevd = stdevd;

% Collocation
% State variables: natural real rate shock, real wage rate

dvec = [linspace(-uncostdevd*4,0,17)'; linspace(0.1,uncostdevd*4,8)'];
wvec = linspace(-0.8,0.08,19)';
fspace = fundefn('spli',[],[],[],3,dvec,wvec);

tol = 1.49*10^(-8); % tolerance crition for iteration

scoord = funnode(fspace); % collocation nodes
snodes = gridmake(scoord);
Phi = funbas(fspace,snodes);

upd = 0.5; % Updating parameter to avoid overshooting
up = 0.2;  

% Gaussian Quadrature 
[epsilons,ws] = qnwnorm(7,0,stdevd^2); 

lambdaGvec = (0.0005:0.0005:0.02)';
loss = zeros(length(lambdaGvec),1);
Closs = zeros(length(lambdaGvec),1);
cpvec = zeros(length(snodes),length(lambdaGvec));
cyvec = zeros(length(snodes),length(lambdaGvec));
cgvec = zeros(length(snodes),length(lambdaGvec));
cpWvec = zeros(length(snodes),length(lambdaGvec));
cwvec = zeros(length(snodes),length(lambdaGvec));
civec = zeros(length(snodes),length(lambdaGvec));
cphiwvec = zeros(length(snodes),length(lambdaGvec));
Ep_wvec = zeros(length(snodes),length(lambdaGvec));
Ey_wvec = zeros(length(snodes),length(lambdaGvec));
Eg_wvec = zeros(length(snodes),length(lambdaGvec));
EpW_wvec = zeros(length(snodes),length(lambdaGvec));
zlbfreq = zeros(length(lambdaGvec),1);
sd_pi = zeros(length(lambdaGvec),1);
sd_y = zeros(length(lambdaGvec),1);
sd_g = zeros(length(lambdaGvec),1);
sd_piW = zeros(length(lambdaGvec),1);
sd_w = zeros(length(lambdaGvec),1);
sd_i = zeros(length(lambdaGvec),1);
sd_phiw = zeros(length(lambdaGvec),1);
m_pi = zeros(length(lambdaGvec),1);
m_y = zeros(length(lambdaGvec),1);
m_g = zeros(length(lambdaGvec),1);
m_piW = zeros(length(lambdaGvec),1);
m_w = zeros(length(lambdaGvec),1);
m_i = zeros(length(lambdaGvec),1);
m_phiw = zeros(length(lambdaGvec),1);
miniw = zeros(length(lambdaGvec),1);
maxiw = zeros(length(lambdaGvec),1);

% Initial guess
pi = 0.078326*snodes(:,2);
y = -0.003546*snodes(:,2);
g = -0.012062*snodes(:,2);
piW = -0.022358*snodes(:,2);
w = 0.899317*snodes(:,2);
i = rstar*ones(length(snodes),1) +1.22*snodes(:,1) +0.069393*snodes(:,2);
phiw = 1/(kappaw*(sigma+eta/(1-gamma))-kappap*gamma/(1-gamma))*(lambda*(y-Gamma*g)+kappap*gamma/(1-gamma)*pi+kappaw*(sigma+eta/(1-gamma))*lambdaW*piW);

cp = Phi\pi;
cy = Phi\y;
cg = Phi\g;
cpW = Phi\piW;
cw = Phi\w;
ci = Phi\i;
cphiw = Phi\phiw;

Ep_w = 0.078326*ones(length(snodes),1); %zeros(length(snodes),1);
Ey_w = -0.003546*ones(length(snodes),1); %zeros(length(snodes),1);
Eg_w = -0.012062*ones(length(snodes),1); %zeros(length(snodes),1);
EpW_w = -0.022358*ones(length(snodes),1); %zeros(length(snodes),1);
% for k=1:1:length(ws);
%     df = rho*snodes(:,1)+ones(length(snodes),1)*epsilons(k,:);
%     Ep_w = Ep_w + ws(k)*funeval(cp,fspace,[df w],[0 1]); % (cp[A(j),b(j+1)]-cp[A(j),b(j)])/h_b
%     Ey_w = Ey_w + ws(k)*funeval(cy,fspace,[df w],[0 1]);
%     Eg_w = Eg_w + ws(k)*funeval(cg,fspace,[df w],[0 1]);
%     EpW_w = EpW_w + ws(k)*funeval(cpW,fspace,[df w],[0 1]);
% end; 
      
load polfuncoef_zlb.mat;
% load Results_grid_search_pref_only.mat;
% pos = 1;
% cp = cpvec(:,pos);
% cy = cyvec(:,pos);
% cg = cgvec(:,pos);
% cpW = cpWvec(:,pos);
% ci = civec(:,pos);
% cw = cwvec(:,pos);
% cphiw = cphiwvec(:,pos);
% Ep_w = Ep_wvec(:,pos);
% Ey_w = Ey_wvec(:,pos);
% Eg_w = Eg_wvec(:,pos);
% EpW_w = EpW_wvec(:,pos);

% Grid of lambdaGs
disp('Number of iterations');
disp(length(lambdaGvec));
disp(' ');
disp('Iteration number:');


for q = 1:length(lambdaGvec);

if q==21;
eval(['save Results_lambdaG' int2str(q-1) '.mat Closs loss zlbfreq sd_pi sd_y sd_g sd_piW sd_w sd_i sd_phiw m_pi m_y m_g m_piW m_w m_i m_phiw maxiw miniw cpvec cyvec cgvec cpWvec cwvec civec cphiwvec Ep_wvec Ey_wvec Eg_wvec EpW_wvec;']);
end;

lambdaG = lambdaGvec(q);
params.lambdaG = lambdaG;
disp([q lambdaG]);  
    
Ep_wnew = zeros(length(snodes),1);
Ey_wnew = zeros(length(snodes),1);
Eg_wnew = zeros(length(snodes),1);
EpW_wnew = zeros(length(snodes),1);
Ep_wold = 1000*ones(length(snodes),1);
Ey_wold = 1000*ones(length(snodes),1);
Eg_wold = 1000*ones(length(snodes),1);
EpW_wold = 1000*ones(length(snodes),1);

while norm([Ep_wnew-Ep_wold;Ey_wnew-Ey_wold;Eg_wnew-Eg_wold;EpW_wnew-EpW_wold],inf) > 1.49*10^(-8);

changepartial = (norm([Ep_wnew-Ep_wold;Ey_wnew-Ey_wold;Eg_wnew-Eg_wold;EpW_wnew-EpW_wold],inf));
   
Ep_wold = Ep_w;
Ey_wold = Ey_w;
Eg_wold = Eg_w;
EpW_wold = EpW_w;

    
stopall = 0;
for it = 1:maxit;
    cpold = cp;
    cyold = cy;
    cgold = cg;
    cpWold = cpW;
    cwold = cw;
    ciold = ci;
    cphiwold = cphiw;
    
    [pi,y,g,piW,w,i,phiw] = NKMsolve(snodes,cp,cy,cg,cpW,cw,ci,cphiw,epsilons,ws,fspace,params,Ep_w,Ey_w,Eg_w,EpW_w);
                                    

    % Update policy functions (Note, that I use linear splines)
    cpnew = Phi\pi;
    cynew = Phi\y;
    cgnew = Phi\g;
    cpWnew = Phi\piW;
    cwnew = Phi\w;
    cinew = Phi\i;
    cphiwnew = Phi\phiw;
    
    cp = upd*cpnew+(1-upd)*cpold;
    cy = upd*cynew+(1-upd)*cyold;
    cg = upd*cgnew+(1-upd)*cgold;
    cpW = upd*cpWnew+(1-upd)*cpWold;
    cw = upd*cwnew+(1-upd)*cwold;
    ci = upd*cinew+(1-upd)*ciold;
    cphiw = upd*cphiwnew+(1-upd)*cphiwold;
    
    change = norm([cpnew-cpold;cynew-cyold;cgnew-cgold;cpWnew-cpWold;cwnew-cwold;cinew-ciold;cphiwnew-cphiwold],inf);

    if change > 50, disp('Algorithm does not converge.'), stopall = 1; break, end;
    if isnan(change) ==1, disp('Algorithm does not converge.'), stopall = 1; break, end;
    
    if change < tol, break, end;
    
end;

Ep_w = zeros(length(snodes),1);
Ey_w = zeros(length(snodes),1);
Eg_w = zeros(length(snodes),1);
EpW_w = zeros(length(snodes),1);

waux = funeval(cw,fspace,snodes);

for k=1:1:length(ws);
    df = rho*snodes(:,1)+ones(length(snodes),1)*epsilons(k,:);
    Ep_w = Ep_w + ws(k)*funeval(cp,fspace,[df waux],[0 1]); % (cp[d(j),i(j+1)]-cp[d(j),i(j)])/h_i
    Ey_w = Ey_w + ws(k)*funeval(cy,fspace,[df waux],[0 1]);
    Eg_w = Eg_w + ws(k)*funeval(cg,fspace,[df waux],[0 1]);
    EpW_w = EpW_w + ws(k)*funeval(cpW,fspace,[df waux],[0 1]);
end;

Ep_wnew = Ep_w;
Ey_wnew = Ey_w;
Eg_wnew = Eg_w;
EpW_wnew = EpW_w;

Ep_w = up*Ep_wnew+(1-up)*Ep_wold;    
Ey_w = up*Ey_wnew+(1-up)*Ey_wold;
Eg_w = up*Eg_wnew+(1-up)*Eg_wold;
EpW_w = up*EpW_wnew+(1-up)*EpW_wold;
end;


if stopall == 0;
    
cpvec(:,q) = cp;
cyvec(:,q) = cy;
cgvec(:,q) = cg;
cpWvec(:,q) = cpW;
cwvec(:,q) = cw;
civec(:,q) = ci;
cphiwvec(:,q) = cphiw;
Ep_wvec(:,q) = Ep_w;
Ey_wvec(:,q) = Ey_w;
Eg_wvec(:,q) = Eg_w;
EpW_wvec(:,q) = EpW_w;

    
%% Simulation 
nobs = 2000;
T = 1000; 
buin = 50;

[std_pi,std_y,std_g,std_piW,std_w,std_i,std_phiw,...
    mean_pi,mean_y,mean_g,mean_piW,mean_w,mean_i,mean_phiw,...
    lobo,simloss,maxima,minima] = ...
NKMsimul(cp,cy,cg,cpW,cw,ci,cphiw,fspace,T,buin,nobs,params,cufo);


sd_pi(q) = mean(std_pi);
sd_y(q) = mean(std_y);
sd_g(q) = mean(std_g);
sd_piW(q) = mean(std_piW);
sd_w(q) = mean(std_w);
sd_i(q) = mean(std_i);
sd_phiw(q) = mean(std_phiw);
zlbfreq(q) = (mean(lobo))*100;
m_pi(q) = mean(mean_pi);
m_y(q) = mean(mean_y);
m_g(q) = mean(mean_g);
m_piW(q) = mean(mean_piW);
m_w(q) = mean(mean_w);
m_i(q) = mean(mean_i);
m_phiw(q) = mean(mean_phiw);
miniw(q) = min(minima);
maxiw(q) = max(maxima);


loss(q) = mean(simloss);
% Loss in percent of steady state private consumption 
% Remark I: Devide LOSS by 100^(2) because inflation and output gap are
% expressed in percentage terms.
% Remark II: Multiply by 100 because we want to express the loss in
% percentage terms
Closs(q) = (1/2)*(1-beta)*1/lambda*(sigma+(eta+gamma)/(1-gamma))*loss(q)/(100^(2))*100; 


end;

end;

figure;
plot(lambdaGvec,Closs,'LineWidth',2);
title('Consumption loss');

[Clossmin,pos] = min(Closs);
lambdaGopt = lambdaGvec(pos);

% Save results
save Results_grid_search_pref_only_B.mat Clossmin lambdaGopt pos lambdaGvec Closs loss zlbfreq sd_pi sd_y sd_g sd_piW sd_w sd_i sd_phiw m_pi m_y m_g m_piW m_w m_i m_phiw maxiw miniw cpvec cyvec cgvec cpWvec cwvec civec cphiwvec Ep_wvec Ey_wvec Eg_wvec EpW_wvec;

disp('END');
