function [sdp,sdy,sdi,sdg,mp,my,mi,mg,lbb,simloss,maxi,mini] = NKMsimul(cp,cy,ci,cg,fspace,T,buin,nobs,params)

% Function file to simulate New Keynesian model with ZLB

% Subroutines called:
% funeval (from CompEcon)

load simul_innos.mat;
deps = d_inno'; clear d_inno;

beta=params.beta;
rho = params.rho;
lambda = params.lambda;
lambdaGstar = params.lambdaGstar;
Gamma = params.Gamma;
tau = params.tau;

isim = zeros(nobs,T+buin);
psim0 = zeros(nobs,1);
psim = zeros(nobs,T+buin);
ysim = zeros(nobs,T+buin);
gsim = zeros(nobs,T+buin);
d = zeros(nobs,T+buin);

simloss = zeros(nobs,T);
lbindex = zeros(nobs,T);

betaadd = 1;

d(:,1) = deps(:,1);
psim(:,1) = funeval(cp,fspace,[d(:,1) psim0]);

for j = 2:1:T+buin;
    d(:,j)=rho*d(:,j-1)+deps(:,j);
    isim(:,j) = funeval(ci,fspace,[d(:,j) psim(:,j-1)]);
    psim(:,j) = funeval(cp,fspace,[d(:,j) psim(:,j-1)]);
    ysim(:,j) = funeval(cy,fspace,[d(:,j) psim(:,j-1)]);
    gsim(:,j) = funeval(cg,fspace,[d(:,j) psim(:,j-1)]);
    
    if j > buin;   
        simloss(:,j-buin) = betaadd*((psim(:,j)-tau*psim(:,j-1)).^(2)+lambda*(ysim(:,j)-Gamma*gsim(:,j)).^(2)+lambdaGstar*gsim(:,j).^(2));
        betaadd = betaadd*beta;
        
        [row,col] = find(isim(:,j)<=0.000000001);
        lbindex(row,j) = 1;
    end;
end;

lbb = sum(lbindex,2)*1/T;

vary = var(ysim(:,end-T+1:end),0,2);
varp = var(psim(:,end-T+1:end),0,2);
vari = var(isim(:,end-T+1:end),0,2);
varg = var(gsim(:,end-T+1:end),0,2);

sdy = sqrt(vary);
sdp = sqrt(varp);
sdi = sqrt(vari);
sdg = sqrt(varg);

my = mean(ysim(:,end-T+1:end),2);
mp = mean(psim(:,end-T+1:end),2);
mi = mean(isim(:,end-T+1:end),2);
mg = mean(gsim(:,end-T+1:end),2);

maxi = max(psim(:,end-T+1:end));
mini = min(psim(:,end-T+1:end));

simloss = sum(simloss,2);