function [p,y,i,g] = NKMsolve(s,cp,cy,ci,cg,epsilons,ws,fspace,params,Ep_p,Ey_p,Eg_p)

% Function file to solve New Keynesian model with ZLB using Collocation
% method

% Subroutines called:
% funeval (from CompEcon)

beta = params.beta;
sigma = params.sigma;
kappa = params.kappa;
rho = params.rho;
rstar = params.rstar;
Gamma = params.Gamma;
lambda = params.lambda;
lambdaG = params.lambdaG;
tau = params.tau;


% Use results from previous iteration to initialize i
p = funeval(cp,fspace,s);
y = zeros(length(s),1);
i = zeros(length(s),1);
g = zeros(length(s),1);
    
 % Initialixe epectations functions and value function
Ep = zeros(length(s),1);
Ey = zeros(length(s),1);
Eg = zeros(length(s),1);

for k=1:1:length(ws);
    df = rho*s(:,1)+ones(length(s),1)*epsilons(k,:);
    Ep = Ep + ws(k)*funeval(cp,fspace,[df p]);
    Ey = Ey + ws(k)*funeval(cy,fspace,[df p]);
    Eg = Eg + ws(k)*funeval(cg,fspace,[df p]);
end;


% 4x1 (constants)
B = [0; 1/sigma*rstar; 0; 0];

% 4x1 (predetermined endogenous variables)
C = [tau; 0; tau; 0];

% 4x3 (expectations)
D = [beta 0 0;
    1/sigma 1 -1;
    beta*tau beta*tau*lambda/kappa beta*tau*(lambdaG/((1-Gamma)*kappa)-lambda/kappa*Gamma);
    0 0 0];

% 4x1 (shock)
E = [0; 1; 0; 0];

for j =1:1:length(s);
    OM1 = beta*Ep_p(j)-1-beta*tau;
    OM2 = Ey_p(j)-Eg_p(j)+1/sigma*Ep_p(j);

% 4x4 (free endogenous variables)
    A = [(1+beta*tau) -kappa Gamma*kappa 0;
        0 1 -1 1/sigma;
        (1+beta*tau^2) -OM1*lambda/kappa OM1*(lambda/kappa*Gamma-lambdaG/((1-Gamma)*kappa))-OM2*lambdaG/(1-Gamma) 0;
        0 0 1 0];
    
 Azlb = [(1+beta*tau) -kappa Gamma*kappa 0;
        0 1 -1 1/sigma;
        (1+beta*tau^2) -OM1*lambda/kappa OM1*(lambda/kappa*Gamma-lambdaG/((1-Gamma)*kappa))-OM2*lambdaG/(1-Gamma) 0;
        0 0 0 1];
    
    z = A\B + A\C*s(j,2) + A\D*[Ep(j); Ey(j); Eg(j)] + A\E*s(j,1);

    
    if z(4)<0;
        z = Azlb\B + Azlb\C*s(j,2) + Azlb\D*[Ep(j); Ey(j); Eg(j)] + Azlb\E*s(j,1);
    end;
    
    p(j)= z(1);
    y(j)= z(2);
    g(j)= z(3);
    i(j)= z(4);
end;
